==> llm-docs-0.1.2.tgz: package/docs/api_documentation/model_registry.md <==
---
id: api_documentation-model_registry
---
# Model Registry API Documentation

This document provides comprehensive documentation for the Model Registry API in the Bluefly AI Platform.

## Overview

The Model Registry API allows developers to interact with the model registry, deployment system, and inference services of the Bluefly AI Platform. It provides endpoints for managing models, creating deployments, and running inference.

## Base URL

```
https://api.bluefly.io/v1
```

## Authentication

This API uses JSON Web Tokens (JWT) for authentication. To authenticate your requests, include an Authorization header with a Bearer token.

```
Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9...
```

To obtain a token, make a POST request to the /auth endpoint with your credentials.

## Request Headers

| Header | Value | Description |
|--------|-------|-------------|
| Authorization | Bearer {jwt_token} | Required for authenticated endpoints |
| Content-Type | application/json | Required for endpoints accepting request body |
| Accept | application/json | Specifies the response format |
| X-API-Key | your_api_key | Required for endpoints requiring API key |
| X-Request-ID | request_uuid | Optional request identifier for tracking |

## Endpoints

### List Models

Retrieves a list of available models.

**URL**: `/models`  
**Method**: `GET`  
**Auth required**: Yes

#### Query Parameters

```json
{
  "provider": "openai",  // Optional filter by provider
  "is_active": true,     // Optional filter by active status
  "limit": 20,           // Optional limit (default: 20, max: 100)
  "offset": 0            // Optional pagination offset
}
```

#### Success Response

**Code**: `200 OK`

```json
{
  "models": [
    {
      "id": 1,
      "name": "gpt-4",
      "provider": "openai",
      "base_model": "gpt-4",
      "description": "GPT-4 model with improved reasoning",
      "capabilities": {
        "vision": true,
        "streaming": true,
        "function_calling": true
      },
      "is_active": true,
      "created_at": "2025-01-15T12:30:45Z",
      "updated_at": "2025-03-22T09:15:20Z"
    },
    {
      "id": 2,
      "name": "claude-3-sonnet",
      "provider": "anthropic",
      "base_model": "claude-3",
      "description": "Claude 3 Sonnet model",
      "capabilities": {
        "vision": true,
        "streaming": true,
        "function_calling": true
      },
      "is_active": true,
      "created_at": "2025-02-10T14:25:30Z",
      "updated_at": "2025-04-05T11:40:15Z"
    }
  ],
  "pagination": {
    "total": 15,
    "limit": 20,
    "offset": 0
  }
}
```

#### Error Responses

**Code**: `401 Unauthorized`

```json
{
  "error": "invalid_token",
  "message": "Authentication token is invalid or expired"
}
```

**Code**: `403 Forbidden`

```json
{
  "error": "insufficient_permissions",
  "message": "You do not have permission to access this resource"
}
```

#### Example

```bash
curl -X GET https://api.bluefly.io/v1/models?provider=openai&is_active=true \
  -H "Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9..."
```

### Create Deployment

Creates a new model deployment.

**URL**: `/deployments`  
**Method**: `POST`  
**Auth required**: Yes

#### Request Body

```json
{
  "name": "production-gpt4",
  "model_id": 1,
  "model_version_id": 3,
  "environment": "production",
  "scaling": {
    "min_replicas": 2,
    "max_replicas": 10,
    "target_cpu_utilization": 70
  },
  "config": {
    "timeout_seconds": 60,
    "max_tokens": 8192,
    "temperature": 0.7
  }
}
```

#### Success Response

**Code**: `201 Created`

```json
{
  "id": 42,
  "name": "production-gpt4",
  "model_id": 1,
  "model_version_id": 3,
  "status": "provisioning",
  "environment": "production",
  "endpoint": "https://api.bluefly.io/v1/inference/deployments/production-gpt4",
  "scaling": {
    "min_replicas": 2,
    "max_replicas": 10,
    "target_cpu_utilization": 70
  },
  "config": {
    "timeout_seconds": 60,
    "max_tokens": 8192,
    "temperature": 0.7
  },
  "created_at": "2025-05-30T15:45:30Z",
  "created_by": "user@example.com"
}
```

#### Error Responses

**Code**: `400 Bad Request`

```json
{
  "error": "validation_error",
  "message": "Invalid request data",
  "details": [
    {
      "field": "name",
      "message": "Deployment name must be unique"
    }
  ]
}
```

#### Example

```bash
curl -X POST https://api.bluefly.io/v1/deployments \
  -H "Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9..." \
  -H "Content-Type: application/json" \
  -d '{
    "name": "production-gpt4",
    "model_id": 1,
    "model_version_id": 3,
    "environment": "production",
    "scaling": {
      "min_replicas": 2,
      "max_replicas": 10,
      "target_cpu_utilization": 70
    },
    "config": {
      "timeout_seconds": 60,
      "max_tokens": 8192,
      "temperature": 0.7
    }
  }'
```

## Data Models

### Model

Represents an AI model in the system.

```json
{
  "id": "integer",          // Unique identifier
  "name": "string",         // Model name (unique)
  "provider": "string",     // AI provider (e.g., OpenAI, Anthropic)
  "base_model": "string",   // Base model identifier
  "description": "string",  // Optional model description
  "capabilities": {         // Model capabilities
    "vision": "boolean",
    "streaming": "boolean",
    "function_calling": "boolean"
  },
  "is_active": "boolean",   // Whether the model is active
  "created_at": "string (ISO 8601 datetime)",
  "updated_at": "string (ISO 8601 datetime)"
}
```

### Deployment

Represents a model deployment.

```json
{
  "id": "integer",          // Unique identifier
  "name": "string",         // Deployment name (unique)
  "model_id": "integer",    // Reference to Model ID
  "model_version_id": "integer", // Reference to ModelVersion ID
  "status": "string",       // "provisioning", "running", "error", "stopped"
  "environment": "string",  // "development", "staging", "production"
  "endpoint": "string",     // Inference endpoint URL
  "scaling": {              // Scaling configuration
    "min_replicas": "integer",
    "max_replicas": "integer",
    "target_cpu_utilization": "integer"
  },
  "config": {               // Model configuration
    "timeout_seconds": "integer",
    "max_tokens": "integer",
    "temperature": "number"
  },
  "created_at": "string (ISO 8601 datetime)",
  "created_by": "string"    // User who created the deployment
}
```

## Error Codes

| Code | Error Key | Description |
|------|-----------|-------------|
| 400 | bad_request | The request was malformed or missing required parameters |
| 401 | unauthorized | Authentication is required or credentials are invalid |
| 403 | forbidden | The authenticated user lacks permission for this operation |
| 404 | not_found | The requested resource was not found |
| 422 | validation_error | The request data failed validation |
| 429 | rate_limit_exceeded | Too many requests, rate limit exceeded |
| 500 | server_error | An unexpected server error occurred |

## Rate Limiting

The API implements rate limiting to ensure fair usage and system stability:

- Authentication endpoints: 10 requests per minute per IP address
- Standard endpoints: 60 requests per minute per authenticated user
- Bulk operations: 10 requests per minute per authenticated user

When rate limits are exceeded, the API returns a 429 Too Many Requests response with a Retry-After header indicating the number of seconds to wait before retrying.

## Versioning

The API uses URL-based versioning with the format "/v{version_number}".

- Current stable version: v1
- Beta features: v2 (subject to change)

Breaking changes are only introduced in new major versions. Minor updates and bug fixes are applied to the current version without changing the API contract.

## Pagination

List endpoints support pagination using the following query parameters:

- "page": Page number (1-based, default: 1)
- "per_page": Number of items per page (default: 20, max: 100)

Response includes pagination metadata:

```json
{
  "data": [...],
  "pagination": {
    "total_items": 243,
    "total_pages": 13,
    "current_page": 1,
    "per_page": 20,
    "next_page": 2,
    "prev_page": null
  }
}
```

For large datasets, consider using cursor-based pagination with the "cursor" and "limit" parameters.

## Examples

### Complete Workflow Example

```javascript
// Complete model deployment and inference workflow
async function deployAndRunInference() {
  // 1. Authenticate
  const authResponse = await fetch('https://api.bluefly.io/v1/auth', {
    method: 'POST',
    headers: { 'Content-Type': 'application/json' },
    body: JSON.stringify({
      username: 'user@example.com',
      password: 'secure_password'
    })
  });
  
  const authData = await authResponse.json();
  const token = authData.token;
  
  // 2. List available models
  const modelsResponse = await fetch('https://api.bluefly.io/v1/models?is_active=true', {
    headers: {
      'Authorization': 'Bearer ' + token,
      'Accept': 'application/json'
    }
  });
  
  const modelsData = await modelsResponse.json();
  const selectedModel = modelsData.models.find(m => m.name === 'gpt-4');
  
  // 3. Create deployment
  const deploymentResponse = await fetch('https://api.bluefly.io/v1/deployments', {
    method: 'POST',
    headers: {
      'Authorization': 'Bearer ' + token,
      'Content-Type': 'application/json'
    },
    body: JSON.stringify({
      name: 'my-gpt4-deployment',
      model_id: selectedModel.id,
      model_version_id: 3,
      environment: 'development',
      scaling: {
        min_replicas: 1,
        max_replicas: 3,
        target_cpu_utilization: 70
      },
      config: {
        timeout_seconds: 30,
        max_tokens: 4096,
        temperature: 0.7
      }
    })
  });
  
  const deploymentData = await deploymentResponse.json();
  
  // 4. Wait for deployment to be ready
  let deploymentStatus = deploymentData.status;
  let deploymentDetails;
  
  while (deploymentStatus === 'provisioning') {
    await new Promise(resolve => setTimeout(resolve, 5000));
    
    const statusResponse = await fetch(`https://api.bluefly.io/v1/deployments/${deploymentData.id}`, {
      headers: {
        'Authorization': 'Bearer ' + token,
        'Accept': 'application/json'
      }
    });
    
    deploymentDetails = await statusResponse.json();
    deploymentStatus = deploymentDetails.status;
  }
  
  // 5. Run inference
  const inferenceResponse = await fetch(deploymentDetails.endpoint, {
    method: 'POST',
    headers: {
      'Authorization': 'Bearer ' + token,
      'Content-Type': 'application/json'
    },
    body: JSON.stringify({
      prompt: "Explain quantum computing in simple terms",
      max_tokens: 500,
      temperature: 0.7
    })
  });
  
  const inferenceResult = await inferenceResponse.json();
  
  return inferenceResult;
}
```

## Best Practices

1. Always validate your API tokens before making requests
2. Implement proper error handling for all API calls
3. Use exponential backoff for retrying failed requests
4. Cache frequently accessed resources to reduce API calls
5. Use webhook notifications for long-running operations
6. Implement proper pagination for large data sets
7. Follow the principle of least privilege when requesting API permissions

## Related Documentation

- [Model Registry Database Schema](../database_schemas/llm_platform)
- [Deployment System](../core_documentation/deployment_system)

## References

- [OpenAPI Specification](https://example.com/openapi-spec)
- [JSON Schema](https://example.com/json-schema)
- [API Security Best Practices](https://example.com/api-security)

---

**Version**: 1.0.0  
**Last Updated**: May 31, 2025  
**Status**: Draft

==> llm-docs-0.1.5.tgz: package/docs/api_documentation/model_registry.md <==
---
id: api_documentation-model_registry
---
# Model Registry API Documentation

This document provides comprehensive documentation for the Model Registry API in the LLM Platform AI Platform.

## Overview

The Model Registry API allows developers to interact with the model registry, deployment system, and inference services of the LLM Platform AI Platform. It provides endpoints for managing models, creating deployments, and running inference.

## Base URL

```
https://api.bluefly.io/v1
```

## Authentication

This API uses JSON Web Tokens (JWT) for authentication. To authenticate your requests, include an Authorization header with a Bearer token.

```
Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9...
```

To obtain a token, make a POST request to the /auth endpoint with your credentials.

## Request Headers

| Header | Value | Description |
|--------|-------|-------------|
| Authorization | Bearer `{jwt_token}` | Required for authenticated endpoints |
| Content-Type | application/json | Required for endpoints accepting request body |
| Accept | application/json | Specifies the response format |
| X-API-Key | your_api_key | Required for endpoints requiring API key |
| X-Request-ID | request_uuid | Optional request identifier for tracking |

## Endpoints

### List Models

Retrieves a list of available models.

**URL**: `/models`  
**Method**: `GET`  
**Auth required**: Yes

#### Query Parameters

```json
{
  "provider": "openai",  // Optional filter by provider
  "is_active": true,     // Optional filter by active status
  "limit": 20,           // Optional limit (default: 20, max: 100)
  "offset": 0            // Optional pagination offset
}
```

#### Success Response

**Code**: `200 OK`

```json
{
  "models": [
    {
      "id": 1,
      "name": "gpt-4",
      "provider": "openai",
      "base_model": "gpt-4",
      "description": "GPT-4 model with improved reasoning",
      "capabilities": {
        "vision": true,
        "streaming": true,
        "function_calling": true
      },
      "is_active": true,
      "created_at": "2025-01-15T12:30:45Z",
      "updated_at": "2025-03-22T09:15:20Z"
    },
    {
      "id": 2,
      "name": "claude-3-sonnet",
      "provider": "anthropic",
      "base_model": "claude-3",
      "description": "Claude 3 Sonnet model",
      "capabilities": {
        "vision": true,
        "streaming": true,
        "function_calling": true
      },
      "is_active": true,
      "created_at": "2025-02-10T14:25:30Z",
      "updated_at": "2025-04-05T11:40:15Z"
    }
  ],
  "pagination": {
    "total": 15,
    "limit": 20,
    "offset": 0
  }
}
```

#### Error Responses

**Code**: `401 Unauthorized`

```json
{
  "error": "invalid_token",
  "message": "Authentication token is invalid or expired"
}
```

**Code**: `403 Forbidden`

```json
{
  "error": "insufficient_permissions",
  "message": "You do not have permission to access this resource"
}
```

#### Example

```bash
curl -X GET https://api.bluefly.io/v1/models?provider=openai&is_active=true \
  -H "Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9..."
```

### Create Deployment

Creates a new model deployment.

**URL**: `/deployments`  
**Method**: `POST`  
**Auth required**: Yes

#### Request Body

```json
{
  "name": "production-gpt4",
  "model_id": 1,
  "model_version_id": 3,
  "environment": "production",
  "scaling": {
    "min_replicas": 2,
    "max_replicas": 10,
    "target_cpu_utilization": 70
  },
  "config": {
    "timeout_seconds": 60,
    "max_tokens": 8192,
    "temperature": 0.7
  }
}
```

#### Success Response

**Code**: `201 Created`

```json
{
  "id": 42,
  "name": "production-gpt4",
  "model_id": 1,
  "model_version_id": 3,
  "status": "provisioning",
  "environment": "production",
  "endpoint": "https://api.bluefly.io/v1/inference/deployments/production-gpt4",
  "scaling": {
    "min_replicas": 2,
    "max_replicas": 10,
    "target_cpu_utilization": 70
  },
  "config": {
    "timeout_seconds": 60,
    "max_tokens": 8192,
    "temperature": 0.7
  },
  "created_at": "2025-05-30T15:45:30Z",
  "created_by": "user@example.com"
}
```

#### Error Responses

**Code**: `400 Bad Request`

```json
{
  "error": "validation_error",
  "message": "Invalid request data",
  "details": [
    {
      "field": "name",
      "message": "Deployment name must be unique"
    }
  ]
}
```

#### Example

```bash
curl -X POST https://api.bluefly.io/v1/deployments \
  -H "Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9..." \
  -H "Content-Type: application/json" \
  -d '{
    "name": "production-gpt4",
    "model_id": 1,
    "model_version_id": 3,
    "environment": "production",
    "scaling": {
      "min_replicas": 2,
      "max_replicas": 10,
      "target_cpu_utilization": 70
    },
    "config": {
      "timeout_seconds": 60,
      "max_tokens": 8192,
      "temperature": 0.7
    }
  }'
```

## Data Models

### Model

Represents an AI model in the system.

```json
{
  "id": "integer",          // Unique identifier
  "name": "string",         // Model name (unique)
  "provider": "string",     // AI provider (e.g., OpenAI, Anthropic)
  "base_model": "string",   // Base model identifier
  "description": "string",  // Optional model description
  "capabilities": {         // Model capabilities
    "vision": "boolean",
    "streaming": "boolean",
    "function_calling": "boolean"
  },
  "is_active": "boolean",   // Whether the model is active
  "created_at": "string (ISO 8601 datetime)",
  "updated_at": "string (ISO 8601 datetime)"
}
```

### Deployment

Represents a model deployment.

```json
{
  "id": "integer",          // Unique identifier
  "name": "string",         // Deployment name (unique)
  "model_id": "integer",    // Reference to Model ID
  "model_version_id": "integer", // Reference to ModelVersion ID
  "status": "string",       // "provisioning", "running", "error", "stopped"
  "environment": "string",  // "development", "staging", "production"
  "endpoint": "string",     // Inference endpoint URL
  "scaling": {              // Scaling configuration
    "min_replicas": "integer",
    "max_replicas": "integer",
    "target_cpu_utilization": "integer"
  },
  "config": {               // Model configuration
    "timeout_seconds": "integer",
    "max_tokens": "integer",
    "temperature": "number"
  },
  "created_at": "string (ISO 8601 datetime)",
  "created_by": "string"    // User who created the deployment
}
```

## Error Codes

| Code | Error Key | Description |
|------|-----------|-------------|
| 400 | bad_request | The request was malformed or missing required parameters |
| 401 | unauthorized | Authentication is required or credentials are invalid |
| 403 | forbidden | The authenticated user lacks permission for this operation |
| 404 | not_found | The requested resource was not found |
| 422 | validation_error | The request data failed validation |
| 429 | rate_limit_exceeded | Too many requests, rate limit exceeded |
| 500 | server_error | An unexpected server error occurred |

## Rate Limiting

The API implements rate limiting to ensure fair usage and system stability:

- Authentication endpoints: 10 requests per minute per IP address
- Standard endpoints: 60 requests per minute per authenticated user
- Bulk operations: 10 requests per minute per authenticated user

When rate limits are exceeded, the API returns a 429 Too Many Requests response with a Retry-After header indicating the number of seconds to wait before retrying.

## Versioning

The API uses URL-based versioning with the format "/v\{version_number\}".

- Current stable version: v1
- Beta features: v2 (subject to change)

Breaking changes are only introduced in new major versions. Minor updates and bug fixes are applied to the current version without changing the API contract.

## Pagination

List endpoints support pagination using the following query parameters:

- "page": Page number (1-based, default: 1)
- "per_page": Number of items per page (default: 20, max: 100)

Response includes pagination metadata:

```json
{
  "data": [...],
  "pagination": {
    "total_items": 243,
    "total_pages": 13,
    "current_page": 1,
    "per_page": 20,
    "next_page": 2,
    "prev_page": null
  }
}
```

For large datasets, consider using cursor-based pagination with the "cursor" and "limit" parameters.

## Examples

### Complete Workflow Example

```javascript
// Complete model deployment and inference workflow
async function deployAndRunInference() {
  // 1. Authenticate
  const authResponse = await fetch('https://api.bluefly.io/v1/auth', {
    method: 'POST',
    headers: { 'Content-Type': 'application/json' },
    body: JSON.stringify({
      username: 'user@example.com',
      password: 'secure_password'
    })
  });
  
  const authData = await authResponse.json();
  const token = authData.token;
  
  // 2. List available models
  const modelsResponse = await fetch('https://api.bluefly.io/v1/models?is_active=true', {
    headers: {
      'Authorization': 'Bearer ' + token,
      'Accept': 'application/json'
    }
  });
  
  const modelsData = await modelsResponse.json();
  const selectedModel = modelsData.models.find(m => m.name === 'gpt-4');
  
  // 3. Create deployment
  const deploymentResponse = await fetch('https://api.bluefly.io/v1/deployments', {
    method: 'POST',
    headers: {
      'Authorization': 'Bearer ' + token,
      'Content-Type': 'application/json'
    },
    body: JSON.stringify({
      name: 'my-gpt4-deployment',
      model_id: selectedModel.id,
      model_version_id: 3,
      environment: 'development',
      scaling: {
        min_replicas: 1,
        max_replicas: 3,
        target_cpu_utilization: 70
      },
      config: {
        timeout_seconds: 30,
        max_tokens: 4096,
        temperature: 0.7
      }
    })
  });
  
  const deploymentData = await deploymentResponse.json();
  
  // 4. Wait for deployment to be ready
  let deploymentStatus = deploymentData.status;
  let deploymentDetails;
  
  while (deploymentStatus === 'provisioning') {
    await new Promise(resolve => setTimeout(resolve, 5000));
    
    const statusResponse = await fetch(`https://api.bluefly.io/v1/deployments/${deploymentData.id}`, {
      headers: {
        'Authorization': 'Bearer ' + token,
        'Accept': 'application/json'
      }
    });
    
    deploymentDetails = await statusResponse.json();
    deploymentStatus = deploymentDetails.status;
  }
  
  // 5. Run inference
  const inferenceResponse = await fetch(deploymentDetails.endpoint, {
    method: 'POST',
    headers: {
      'Authorization': 'Bearer ' + token,
      'Content-Type': 'application/json'
    },
    body: JSON.stringify({
      prompt: "Explain quantum computing in simple terms",
      max_tokens: 500,
      temperature: 0.7
    })
  });
  
  const inferenceResult = await inferenceResponse.json();
  
  return inferenceResult;
}
```

## Best Practices

1. Always validate your API tokens before making requests
2. Implement proper error handling for all API calls
3. Use exponential backoff for retrying failed requests
4. Cache frequently accessed resources to reduce API calls
5. Use webhook notifications for long-running operations
6. Implement proper pagination for large data sets
7. Follow the principle of least privilege when requesting API permissions

## Related Documentation

- [Model Registry Database Schema](../database_schemas/llm_platform)
- [Deployment System](../core_documentation/deployment_system)

## References

- [OpenAPI Specification](https://example.com/openapi-spec)
- [JSON Schema](https://example.com/json-schema)
- [API Security Best Practices](https://example.com/api-security)

---

**Version**: 1.0.0  
**Last Updated**: May 31, 2025  
**Status**: Draft
